from functools import update_wrapper
from importlib import import_module

from django.contrib import admin
from django.contrib.admin.sites import AdminSite
from django.conf import settings
from django.conf.urls import patterns, url
from django.template.response import TemplateResponse

from ..admin import GipsyMenu, ChildrenInline
from .models import GipsyDashboardMenu


class ChildrenDashboardInline(ChildrenInline):
    model = GipsyDashboardMenu
    exclude = ('icon',)


class GipsyDashboardMenuAdmin(GipsyMenu):
    inlines = [ChildrenDashboardInline]
    exclude = ('url', 'parent',)


class GipsyAdminSite(AdminSite):
    def get_urls(self):
        default_urlpatterns = super(GipsyAdminSite, self).get_urls()

        def wrap(view, cacheable=False):
            def wrapper(*args, **kwargs):
                return self.admin_view(view, cacheable)(*args, **kwargs)
            return update_wrapper(wrapper, view)

        urlpatterns = patterns(
            '',
            url(r'^$', wrap(self.dashboard), name='index'),
            url(r'^all-apps$',
                wrap(self.index),
                name='all_apps')
        )
        return urlpatterns + default_urlpatterns

    def init_dashboard_class(self):
        dashboard_module = getattr(
            settings,
            'GIPSY_DASHBOARD',
            'gipsy.dashboard.presets.default.DashboardDefault'
        )
        mod, inst = dashboard_module.rsplit('.', 1)
        mod = import_module(mod)
        return getattr(mod, inst)

    def dashboard(self, request):
        """
        Displays the dashboard on the main page and triggers widget
        from the settings.GIPSY_DASHBOARD constant.
        """
        context = dict(
            dashboard=self.init_dashboard_class()(request),
        )
        return TemplateResponse(request, 'admin/dashboard.html', context)

if not hasattr(settings, 'GIPSY_ENABLE_ADMINSITE') or \
   settings.GIPSY_ENABLE_ADMINSITE is not False:
    admin.site = GipsyAdminSite(name='gipsy_admin')
admin.site.register(GipsyDashboardMenu, GipsyDashboardMenuAdmin)



from django import template
from django.db.models import Q

from gipsy.dashboard.models import GipsyDashboardMenu
from gipsy.dashboard.settings import GIPSY_DASHBOARD_URL,\
    GIPSY_VANILLA_INDEX_URL, GIPSY_THEME, GIPSY_DASHBOARD_TITLE,\
    GIPSY_DASHBOARD_CACHE_TIME


register = template.Library()
tag_func = register.inclusion_tag('gipsy/dashboard/widgets/base.html',
                                  takes_context=True)


@register.inclusion_tag('gipsy/dashboard/menu.html',
                        takes_context=True)
def gipsy_dashboard_menu(context, *args, **kwargs):
    """
    This tags manages the display of the admin menu.
    """
    context['items'] = GipsyDashboardMenu.objects.filter(parent__isnull=True)\
        .order_by('order')
    context['active'] = None
    if context['request'].path:
        request_formated = context['request'].get_full_path()[1:]
        active = GipsyDashboardMenu.objects\
            .filter(Q(url=request_formated) | Q(url=request_formated[:-1]))[:1]
    if len(active):
        context['active'] = active[0]
    context['dashboard_url'] = GIPSY_DASHBOARD_URL
    context['vanilla_index_url'] = GIPSY_VANILLA_INDEX_URL
    return context


@register.inclusion_tag('gipsy/dashboard/widgets/active_users.html')
def dashboard_active_users(count=0, title="CURRENTLY ACTIVE USERS",
                           label="CURRENT USERS"):
    return {'count': count, 'title': title, 'label': label}


@register.inclusion_tag('gipsy/dashboard/widgets/item_list.html')
def dashboard_item_list(items, title="MOST RECENT ITEMS"):
    return {'items': items, 'title': title}


@register.simple_tag
def gipsy_theme():
    """
    Returns the theme for the Admin-Interface.
    """
    return GIPSY_THEME


@register.simple_tag
def gipsy_title():
    """
    Returns the Title for the Admin-Interface.
    """
    return GIPSY_DASHBOARD_TITLE


@register.assignment_tag(takes_context=True)
def gipsy_is_popup(context):
    """
    Checks if displayed in a popup. Addition to the django default
    templatetag as some shitty librairiesare not using this logic.
    """
    return (context.get('is_popup', False) or
            context['request'].GET.get('pop', False))


class GipsyDashboardCacheTime(template.Node):
    def __init__(self, varname):
        self.varname = varname

    def __repr__(self):
        return "<GipsyDashboardCachTime Node>"

    def render(self, context):
        context[self.varname] = GIPSY_DASHBOARD_CACHE_TIME
        return ''


@register.tag
def gipsy_dashboard_cache_time(parser, token):
    """
    Retrieves the cache time set in settings

    Usage::

        {% gipsy_dashboard_cache_time as [cache_time] %}
    """
    tokens = token.contents.split()
    if len(tokens) < 2:
        raise template.TemplateSyntaxError(
            "'gipsy_dashboard_cache_time' statements require at least one arguments")
    if tokens[1] != 'as':
        raise template.TemplateSyntaxError(
            "Second argument to 'gipsy_dashboard_cache_time' must be 'as'")

    return GipsyDashboardCacheTime(varname=tokens[2])


def gipsy_dashboard_widget(context, widget, index=None):
    """
    Template tag that renders a given dashboard module
    """
    context.update({
        'template': widget.template,
        'widget': widget,
        'index': index,
    })
    return context
gipsy_dashboard_widget = tag_func(gipsy_dashboard_widget)



from django import template
from django.conf import settings
from django.core.urlresolvers import resolve, reverse

from gipsy.toolbar.models import GipsyToolbarMenu
from gipsy.toolbar.settings import LINK_CONTEXT_NAME, LINK_INCLUDED_MODELS, \
    VERSION_INDICATOR, VERSION_INDICATOR_LOCATION


register = template.Library()


def is_admin(context):
    """Checks if the current request belongs to admin namespace"""
    request = context["request"]
    url = resolve(request.path)
    context['is_admin'] = False
    return url.app_name == 'admin'


@register.inclusion_tag('tags/gipsy_toolbar.html', takes_context=True)
def gipsy_toolbar(context, *args, **kwargs):
    """
    This tags manages the display of the toolbar.
    """
    context['items'] = GipsyToolbarMenu.objects.filter(parent__isnull=True).order_by('order')
    context['logo'] = kwargs.get('logo', '')

    if not context['logo']:
        if hasattr(settings, 'GIPSY_TOOLBAR_LOGO') and settings.GIPSY_TOOLBAR_LOGO:
            context['logo'] = settings.GIPSY_TOOLBAR_LOGO

    context['is_admin'] = is_admin(context)
    return context


@register.assignment_tag(takes_context=True)
def gipsy_toolbar_link(context):
    """
    :param context: django template context
    :return: link to admin edit page for founded object
    """
    obj = context.get(LINK_CONTEXT_NAME, None)
    if obj is None:
        return None

    app_label = obj.__class__._meta.app_label
    model_name = obj.__class__._meta.object_name
    app_ident = u'%s.%s' % (app_label, model_name)

    if LINK_INCLUDED_MODELS is None or app_ident in LINK_INCLUDED_MODELS:
        link = reverse('admin:%s_%s_change' % (
            app_label.lower(), model_name.lower()), args=(obj.pk,))
        return link
    else:
        return None


@register.inclusion_tag('tags/version_indicator.html', takes_context=True)
def gipsy_version_indicator(context, location, css_classes=None):
    context['display_version_indicator'] = False
    if VERSION_INDICATOR and VERSION_INDICATOR_LOCATION == location:
        context['version_label'], context['version_description'] = \
            VERSION_INDICATOR
        context['display_version_indicator'] = True

        if css_classes is not None:
            context['css_classes'] = css_classes

    return context



from django.db import models
from django.utils.translation import ugettext_lazy as _


class GipsyMenu(models.Model):
    """
    Model managing the menu displayed in the toolbar. The menu items reference
    to each other to display a parent and its children
    """
    parent = models.ForeignKey('self', blank=True, null=True)
    name = models.CharField(_('Name'), max_length=255)
    url = models.CharField(_('Url'), blank=True, null=True, max_length=255)
    order = models.PositiveSmallIntegerField('Order')

    class Meta:
        ordering = ['order']
        abstract = True

    def __unicode__(self):
        return self.name

    @property
    def url_has_domain(self):
        return self.url.startswith('http')

    @property
    def path(self):
        return '/' + self.url



from django.db import models
from django.utils.translation import ugettext_lazy as _

from ..models import GipsyMenu


class GipsyDashboardMenu(GipsyMenu):
    icon = models.CharField(_('icon'), max_length=20,
                            help_text=_("Font awesome class \
                                        http://fortawesome.github.io/Font-Awesome/icons/\
                                         ie: fa-circle"))

    @property
    def children(self):
        return GipsyDashboardMenu.objects.filter(parent=self.pk)



# -*- coding: utf-8 -*-


class GipsyToolbarMiddleware(object):
    def process_response(self, request, response):
        """
        Adds a cookie when a user is allowed to access the toolbar. This is usefull to deactivate cache engines
        such as Varnish
        """
        if not hasattr(request, 'user'):
            return response
        if request.user.is_active and request.user.is_staff and not request.COOKIES.get('gt_activated'):
            response.set_cookie(key='gt_activated', value=1)

        if (not request.user.is_active or not request.user.is_staff) and request.COOKIES.get('gt_activated'):
            response.delete_cookie('gt_activated')

        return response



import httplib2
from apiclient.discovery import build
from oauth2client.client import flow_from_clientsecrets
from oauth2client.file import Storage
from oauth2client.tools import run_flow

from django.conf import settings


class GoogleAnalyticsConnector(object):
    """
    Connector class for the google API. This is a very implementation using
    the method described in http://blog.iambob.me/accessing-google-analytics-from-django/
    and in the official API documentation
    https://developers.google.com/analytics/solutions/articles/hello-analytics-api
    """
    CLIENT_SECRETS = settings.GOOGLE_ANALYTICS_CLIENT_SECRETS
    VIEW_ID = settings.GOOGLE_ANALYTICS_VIEW_ID
    TOKEN_FILE_NAME = settings.GOOGLE_ANALYTICS_TOKEN_FILE_NAME
    FLOW = flow_from_clientsecrets(CLIENT_SECRETS, scope='https://www.googleapis.com/auth/analytics.readonly')

    def __init__(self):
        self.service = None

    def _authenticate(self):
        # Retrieve existing credendials
        storage = Storage(self.TOKEN_FILE_NAME)
        credentials = storage.get()

        if credentials is None or credentials.invalid:
            credentials = run_flow(self.FLOW, storage, self.GAFlags())

        return credentials

    def _create_service(self):
        # 1. Create an http object
        http = httplib2.Http()

        # 2. Authorize the http object
        credentials = self._authenticate()
        http = credentials.authorize(http)

        # 3. Build the Analytics Service Object with the authorized http object
        return build('analytics', 'v3', http=http)

    def start_service(self):
        """
        :return: ``this`` Google analytics object with the service set

        start the service which may be used to query the google analytics api
        """
        if not self.service:
            self.service = self._create_service()
        return self

    def query(self, **kwargs):
        kwargs.update({
            'ids': "ga:{id}".format(id=self.VIEW_ID),
        })

        result = self.service.data().ga().get(**kwargs)
        return result

    class GAFlags():
        """
        total hack. If you want to see why, examine apiclient.sample_tools.
        The python bindings as well as the documentation kind've forgot to
        mention this... Which is a big deal... But actually though.
        """
        noauth_local_webserver = True
        logging_level = "DEBUG"



# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'GipsyDashboardMenu'
        db.create_table(u'dashboard_gipsydashboardmenu', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('parent', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['dashboard.GipsyDashboardMenu'], null=True, blank=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('url', self.gf('django.db.models.fields.CharField')(max_length=255, null=True, blank=True)),
            ('order', self.gf('django.db.models.fields.PositiveSmallIntegerField')()),
            ('icon', self.gf('django.db.models.fields.CharField')(max_length=20)),
        ))
        db.send_create_signal(u'dashboard', ['GipsyDashboardMenu'])


    def backwards(self, orm):
        # Deleting model 'GipsyDashboardMenu'
        db.delete_table(u'dashboard_gipsydashboardmenu')


    models = {
        u'dashboard.gipsydashboardmenu': {
            'Meta': {'ordering': "['order']", 'object_name': 'GipsyDashboardMenu'},
            'icon': ('django.db.models.fields.CharField', [], {'max_length': '20'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'order': ('django.db.models.fields.PositiveSmallIntegerField', [], {}),
            'parent': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['dashboard.GipsyDashboardMenu']", 'null': 'True', 'blank': 'True'}),
            'url': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'})
        }
    }

    complete_apps = ['dashboard']